/********************************************************************/
/*																	*/
/*		Demonstracni program pro display FZ012010 verze 4			*/
/*		[c] Filip Zaplata, FEKT VUT v Brne 2010						*/
/*																	*/
/********************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include "FZ012010.h"
#include <string.h>
#include <avr/pgmspace.h>

char addata[660];							//datov pole pro AD pevod
unsigned int disp[16]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; //datov pole displeje
unsigned char STAV=0;						//nastaven procesu (stavu) programu
//grafika polozek menu
unsigned char menu_01[12] PROGMEM={0x3C,0x42,0x3C,0,0x32,0x52,0x4C,0,0x3C,0x42,0x42,0};
unsigned char menu_02[12] PROGMEM={0x7E,0x48,0x40,0,0x7E,0x48,0x40,0,0x40,0x7E,0x40,0};
unsigned char menu_03[12] PROGMEM={0x7E,0x48,0x40,0,0,0x7E,0x42,0x3C,0,0x7E,0x48,0x30};
unsigned char menu_04[12] PROGMEM={0x7E,0x48,0x40,0,0,0x7E,0x10,0x7E,0,0x7E,0x48,0x30};
unsigned char* menu[4];

//*******************************************************************
//*	preformatuje data char na int pro display						*
//*******************************************************************

void cpy_disp(unsigned int *din, char *dout)
{
	char i;

	for (i=0; i<16; i++)
	{
		*dout++=(char)((*din)>>8);
		*dout++=(char)(*din++);
	}
}

//*******************************************************************
//*	probehne text na display										*
//*******************************************************************

int intro(const char* text, unsigned char nxt_st)
{
	static char cnt8=0;
	static int cnt16=0, j;

	if (cnt16==0) 					//naplneni pole addata
	{
		cpy_disp(disp, addata);		//aktualni obraz na disp se ulozi
		strcpy(addata+32, text);	//prida text
		cnt16=kod_text(addata+32);	//prelozi znaky a vrati velikost pole
		j=cnt16+2;
	}
	if (cnt8==0) 					//posun textu
	{
		cnt8=5;						//rychlost posunu
		kopie(j-cnt16, (unsigned char*)addata, j+30, disp, 16);	//prekopiruje data
		cnt16-=2;
		if (cnt16==0) STAV=nxt_st;	//jestlize konec -> dalsi stav
	}
	cnt8--;

	return cnt16;					//vraci pocet cyklu do konce
}

//*******************************************************************
//*	casovace														*
//*******************************************************************

char casovac(const char _CAS)
{
	static char cas=0;

	if (cas==0) cas=_CAS;
	cas--;

	return cas;
}

char casovac0(const char _CAS)
{
	static char cas=0;

	if (cas==0) cas=_CAS;
	cas--;

	return cas;
}

//*******************************************************************
//*	plynuly posun polozek menu										*
//*******************************************************************

char scroll_menu(char m_pos, char n)
{
	static char pos=0;
	char i;
	
	if (n<0) n=-1;				//osetreni spatneho vstupu krok max 1
	if (n>0) n=1;
	if (pos==0) pos=n<<3;		//velikost polozky 8

	if (n<0) pos++;				//zmena pozice obrazu
	if (n>0) pos--;

	for (i=0; i<12; i++) 		//vykresleni menu
	{
		if ((4-pos)<0) disp[i+2]=(pgm_read_byte(menu[m_pos-1]+i))>>(pos-4);
		else disp[i+2]=(pgm_read_byte(menu[m_pos-1]+i))<<(4-pos);
		if (m_pos>1) disp[i+2]+=(pgm_read_byte(menu[m_pos-2]+i))<<(12-pos);
		if (m_pos>2) disp[i+2]+=(pgm_read_byte(menu[m_pos-3]+i))<<(20-pos);
		if (m_pos<4) 
		{
			if ((-4-pos)<0) disp[i+2]+=(pgm_read_byte(menu[m_pos]+i))>>(pos+4); 
			else disp[i+2]+=(pgm_read_byte(menu[m_pos]+i))<<(-pos-4);
		}
		if (m_pos<3) disp[i+2]+=(pgm_read_byte(menu[m_pos]+i))>>(12+pos);
	}
	disp[0]=0x0FF0;				//ukazatel v menu
	disp[1]=0x0;
	disp[14]=0x0;
	disp[15]=0x0FF0;

	return pos;					//vraci pocet cyklu do konce posunu
}

//*******************************************************************
//*	hlavni program													*
//*******************************************************************

int main(void)
{
	unsigned char i, kanal=0, hold=0;
	char menu_pos=1, scroll=0;
	
	init();
	sei();

	menu[0]=&menu_01;						//sestaveni menu
	menu[1]=&menu_02;
	menu[2]=&menu_03;
	menu[3]=&menu_04;
	
	while(1)
	{
//-------------------------------------------vyhodnocen po dokonen AD pevodu
		if ((PRIZNAK>>ADR)==1)
		{
			switch (STAV)
			{
				case 2:						//zobrazeni oscilogramu
					for (i=0; i<16; i++) disp[i]=1<<((unsigned int)(*(addata+i)+8));
				break;
				case 3:						//vypocet spektra
					rFFT(addata, 5);
					for (i=0; i<16; i++) disp[i]=(1<<(absh(&addata[i], &addata[i]+16)>>4))-1;
				break;
				case 4:						//vypocet spektra po decimaci
					IIR_LP01(addata, 340);
					decimace(addata, addata, 320, 10);
					rFFT(addata, 5);
					for (i=0; i<16; i++) disp[i]=(1<<(absh(&addata[i], &addata[i]+16)>>4))-1;
				break;
				case 5:						//vypocet spektra hornich kmitoctu
					IIR_HP01(addata, 52);
					rFFT(addata, 5);
					for (i=0; i<16; i++) disp[i]=(1<<(absh(&addata[i], &addata[i]+16)>>4))-1;
			}
			PRIZNAK&=~(1<<ADR);
		}
//------------------------------------------- vyhodnocen otoen rotanho enkodru
		if ((PRIZNAK>>ROT)==1)
		{
			if ((STAV==1)&&(scroll==0)) 	//pohyb v menu
			{
				if (rotace>0) {menu_pos++; scroll=1;}
				if (rotace<0) {menu_pos--; scroll=-1;}
				if (menu_pos>4) {menu_pos=4; scroll=0;}
				if (menu_pos<1) {menu_pos=1; scroll=0;}
			}
			rotace=0;
			PRIZNAK&=~(1<<ROT);
		}
//------------------------------------------- vyhodnocen zmny stavu tlatek
		if ((PRIZNAK>>TL)==1)
		{
			if ((TLLED_ST&(1<<TL1))!=0) 	//prepinani kanalu
			{
				TLLED_ST&=~(1<<TL1);
				kanal^=1;
			}
			
			if ((TLLED_ST&(1<<TL2))!=0) 	//zmrazeni obrazovky
			{
				TLLED_ST&=~(1<<TL2);
				if (hold==0) {hold=STAV; STAV=6;}
				else {STAV=hold; hold=0;}
			}
			
			if ((TLLED_ST&(1<<TLR))!=0) 	//prechod menu <-> zobrazeni
			{
				TLLED_ST&=~(1<<TLR); 
				if (STAV!=6)
				{	if (STAV==1) STAV=menu_pos+1;
					else STAV=1;
				}
			}
			
			PRIZNAK&=~(1<<TL);
		}
//------------------------------------------- vyhodnocen peteen asovae 6,3ms (159Hz)
		if ((PRIZNAK>>OBR)==1)
		{
			switch (STAV)
			{
				case 0:						//uvodni stav INTRO
					intro("UVODNI TEXT   ", 1);
				break;
				case 1:						//menu + blikani LED
					if (casovac0(80)==0) TLLED^=1<<LED3;
					if (casovac(5)==0) if (scroll_menu(menu_pos, scroll)==0) scroll=0;
				break;
				case 2:						//AD prevod pro oscilogram
					if (casovac(4)==0) adstart(kanal, 16);
				break;
				case 3:						//AD prevod pro FFT
					if (casovac(3)==0) adstart(kanal, 32);
				break;
				case 4:						//AD prevod pro FFT DP
					if (casovac(3)==0) adstart(kanal, 340);
				break;
				case 5:						//AD prevod pro FFT HP
					if (casovac(3)==0) adstart(kanal, 52);
			}
			if (kanal==0) TLLED&=~(1<<LED1); else TLLED|=1<<LED1;	//indikace LED
			if (hold==0) TLLED&=~(1<<LED2); else TLLED|=1<<LED2;
			if (STAV!=1) TLLED&=~(1<<LED3);
			zobr(disp);						//vypis na display
			ledtl();						//test tlacitek a zobrazeni LED
			PRIZNAK&=~(1<<OBR);
		}
//-------------------------------------------
		if (STAV>6) STAV=0;					//oeten zakzanch stav
	}

	return 0;
}
